import json
import subprocess
import os
import time
import shlex

# ~~~~~~

CHECK_PRODUCT_COMPATIBILITY = True

# ~~~~~~

CV_21_01 = [
    'perimica.basesystem e8544bfa25c3dc7f9129a5c35d176ba2cc8da6f4 HEAD'
]

COMPATIBLE_VERSIONS = []
COMPATIBLE_VERSIONS.extend(CV_21_01)

# ~~~~~~

DEFAULT_JSON_FILE_PATH = "/mnt/config/default.json"
NO_COMPATIBLE_INSTALLED_FW_ERROR = "The firmware you are currently using cannot be updated."
P1_CPU_ID = "AM33XX"
P2_CPU_ID = "Marvell Armada 380/385"


# ~~~~~~


def log_and_raise_incompatible_installed_fw(com):
    com.log({'type': 'info', 'code': 111,
             'message': NO_COMPATIBLE_INSTALLED_FW_ERROR,
             'time': int(time.time())})
    raise Exception(NO_COMPATIBLE_INSTALLED_FW_ERROR)


def installed_version_is_updatable():
    return True if get_version_str() in COMPATIBLE_VERSIONS else False


def get_version_str():
    version_file = '/etc/version'
    version_str = ""

    with open(version_file, encoding="utf-8") as vf:
        version_content = vf.readlines()
        version_dict = json.loads("".join(version_content))
        version_str = "{} {}".format(version_dict["Project Name"], version_dict["Product Changeset"])

    return version_str


def assert_file_exists(path):
    if not os.path.isfile(path):
        raise FileNotFoundError(str(path) + " not found")
    else:
        return True


def has_p1():
    return has_cpuinfo(P1_CPU_ID)


def has_p2():
    return has_cpuinfo(P2_CPU_ID)


def has_cpuinfo(id_str):
    ret_val = False
    try:
        subprocess.check_output("cat /proc/cpuinfo | grep -q '" + id_str + "'", shell=True)
        ret_val = True
    except subprocess.CalledProcessError as err:
        pass

    return ret_val


def switch_root_partition(com, next_image):
    com.log({'type': 'info', 'code': 119, 'message': 'Switch linux partition', 'time': int(time.time())})

    subprocess.check_output(
        "echo -e 'rootpart " + next_image + "\nupdate_status 1\nlinux_boot_mode 2\nbootlimit' | fw_setenv -s -",
        shell=True)


def unmount_temp_mounts():
    subprocess.check_output(shlex.split("umount ./mnt/boot"))
    subprocess.check_output(shlex.split("umount ./mnt/rfs"))
    subprocess.check_output(shlex.split("umount ./mnt/user"))


def reboot_mica(com):
    com.log({'type': 'info', 'code': 120, 'message': 'Reboot', 'time': int(time.time())})
    # TODO synchronous rpc call
    time.sleep(3)
    os.system("reboot")
